@@ -1,5 +1,4 @@
-// -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; indent-tabs-mode: nil; -*-
-//
+
 // Module.h: Rcpp R/C++ interface class library -- Rcpp modules
 //
 // Copyright (C) 2010 - 2012 Dirk Eddelbuettel and Romain Francois
@@ -23,6 +22,7 @@
 #define Rcpp_Module_h
 
 #include <Rcpp/config.h>
+#include <Rcpp/internal/call.h>
 
 namespace Rcpp{
 
@@ -85,10 +85,58 @@ namespace Rcpp{
 
 #include <Rcpp/module/CppFunction.h>
 #include <Rcpp/module/get_return_type.h>
-#include <Rcpp/module/Module_generated_get_signature.h>
 
-    // templates CppFunction0, ..., CppFunction65
-#include <Rcpp/module/Module_generated_CppFunction.h>
+#if defined(HAS_VARIADIC_TEMPLATES)
+namespace Rcpp {
+    template <typename RESULT_TYPE, typename... T>
+    inline void signature(std::string& s, const char* name) {
+        s.clear();
+        s += get_return_type<RESULT_TYPE>() + " " + name + "(";
+        int n = sizeof...(T);
+        int i = 0;
+        // Using initializer list as c++11 implementation of a fold expression
+        (void)std::initializer_list<int>{
+            (s += get_return_type<T>(), s += (++i == n ? "" : ", "), 0)... };
+        s += ")";
+    }
+
+    template <typename RESULT_TYPE, typename... T>
+    class CppFunctionN : public CppFunction {
+        public:
+            CppFunctionN(RESULT_TYPE (*fun)(T...), const char* docstring = 0) : CppFunction(docstring), ptr_fun(fun) {}
+
+            SEXP operator()(SEXP* args) {
+                BEGIN_RCPP
+                return call<decltype(ptr_fun), RESULT_TYPE, T...>(ptr_fun, args);
+                END_RCPP
+            }
+
+            inline int nargs() { return sizeof...(T); }
+            inline bool is_void() { return std::is_void<RESULT_TYPE>::value; }
+            inline void signature(std::string& s, const char* name) { Rcpp::signature<RESULT_TYPE, T...>(s, name); }
+            inline DL_FUNC get_function_ptr() { return (DL_FUNC)ptr_fun; }
+
+        private:
+            RESULT_TYPE (*ptr_fun)(T...);
+    };
+
+    template <typename RESULT_TYPE, typename... T>
+    class CppFunction_WithFormalsN : public CppFunctionN<RESULT_TYPE, T...> {
+        public:
+            CppFunction_WithFormalsN(RESULT_TYPE (*fun)(T...), Rcpp::List formals_, const char* docstring = 0) :
+                CppFunctionN<RESULT_TYPE, T...>(fun, docstring), formals(formals_) {}
+
+            SEXP get_formals() { return formals; }
+
+        private:
+            Rcpp::List formals;
+    };
+}
+#else
+    #include <Rcpp/module/Module_generated_get_signature.h>
+        // templates CppFunction0, ..., CppFunction65
+    #include <Rcpp/module/Module_generated_CppFunction.h>
+#endif
 #include <Rcpp/module/class_Base.h>
 #include <Rcpp/module/Module.h>
 
@@ -129,12 +177,86 @@ namespace Rcpp{
     private:
         ParentMethod* parent_method_pointer ;
     } ;
+#if defined(HAS_VARIADIC_TEMPLATES)
+    template <typename... T>
+    inline void ctor_signature(std::string& s, const std::string& classname) {
+        s.assign(classname);
+        s += "(";
+        int n = sizeof...(T);
+        int i = 0;
+        // Using initializer list as c++11 implementation of a fold expression
+        (void)std::initializer_list<int>{
+            (s += get_return_type<T>(), s += (++i == n ? "" : ", "), 0)... };
+        s += ")";
+    }
+    template <typename Class>
+    class Constructor_Base {
+    public:
+        virtual Class* get_new( SEXP* args, int nargs ) = 0 ;
+        virtual int nargs() = 0 ;
+        virtual void signature(std::string& s, const std::string& class_name) = 0 ;
+    } ;
 
+    template <typename Class, typename... T>
+    class Constructor: public Constructor_Base<Class> {
+    public:
+        virtual Class* get_new( SEXP* args, int nargs ){
+            return get_new_impl(args, nargs, traits::make_index_sequence<sizeof...(T)>());
+        }
+        virtual int nargs(){ return sizeof...(T) ; }
+        virtual void signature(std::string& s, const std::string& class_name ){
+            ctor_signature<T...>(s, class_name) ;
+        }
+
+    private:
+        template <int... Is>
+        Class* get_new_impl(SEXP* args, int nargs, traits::index_sequence<Is...>) {
+            return new Class( as<T>(args[Is])... ) ;
+        }
+    };
+
+    template <typename Class>
+    class Factory_Base {
+    public:
+        virtual Class* get_new( SEXP* args, int nargs ) = 0 ;
+        virtual int nargs() = 0 ;
+        virtual void signature(std::string& s, const std::string& class_name) = 0 ;
+    } ;
+
+    template <typename Class, typename... T>
+    class Factory : public Factory_Base<Class> {
+    public:
+        Factory( Class* (*fun)(T...) ) : ptr_fun(fun){}
+        virtual Class* get_new( SEXP* args, int nargs ){
+            return get_new( args, traits::make_index_sequence<sizeof...(T)>() ) ;
+        }
+        virtual int nargs(){ return sizeof...(T) ; }
+        virtual void signature(std::string& s, const std::string& class_name ){
+            ctor_signature<T...>(s, class_name) ;
+        }
+    private:
+        template<int... I>
+        Class* get_new( SEXP* args, traits::index_sequence<I...> ){
+            return ptr_fun( bare_as<T>(args[I])... ) ;
+        }
+        Class* (*ptr_fun)(T...) ;
+    } ;
+
+    inline bool yes( SEXP* /*args*/, int /* nargs */ ){
+        return true ;
+    }
+
+    template<int n>
+    bool yes_arity( SEXP* /* args */ , int nargs){
+        return nargs == n ;
+    }
+
+#else
 #include <Rcpp/module/Module_generated_ctor_signature.h>
 #include <Rcpp/module/Module_generated_Constructor.h>
 #include <Rcpp/module/Module_generated_Factory.h>
-
 #include <Rcpp/module/Module_generated_class_signature.h>
+#endif
 
     typedef bool (*ValidConstructor)(SEXP*,int) ;
     typedef bool (*ValidMethod)(SEXP*,int) ;
@@ -229,7 +351,7 @@ namespace Rcpp{
         typedef std::vector<signed_method_class*> vec_signed_method ;
 
         S4_CppOverloadedMethods( vec_signed_method* m, const XP_Class& class_xp, const char* name, std::string& buffer ) : Reference( "C++OverloadedMethods" ){
-            int n = m->size() ;
+            int n = static_cast<int>(m->size()) ;
             Rcpp::LogicalVector voidness(n), constness(n) ;
             Rcpp::CharacterVector docstrings(n), signatures(n) ;
             Rcpp::IntegerVector nargs(n) ;
@@ -259,8 +381,70 @@ namespace Rcpp{
 
     } ;
 
-#include <Rcpp/module/Module_generated_CppMethod.h>
-#include <Rcpp/module/Module_generated_Pointer_CppMethod.h>
+#if defined(HAS_VARIADIC_TEMPLATES)
+    template <bool IsConst,typename Class, typename RESULT_TYPE, typename... T>
+    class CppMethodImplN : public CppMethod<Class> {
+    public:
+        typedef typename std::conditional<IsConst, RESULT_TYPE (Class::*)(T...) const,
+                                            RESULT_TYPE (Class::*)(T...)>::type Method;
+        typedef CppMethod<Class> method_class;
+        typedef typename Rcpp::traits::remove_const_and_reference<RESULT_TYPE>::type CLEANED_RESULT_TYPE;
+
+        CppMethodImplN(Method m) : method_class(), met(m) {}
+        SEXP operator()(Class* object, SEXP* args) {
+            // Can't pass pointer to member function directly to `call()`, so wrap it in a lambda
+            auto f = [&object, this](T... cpp_args) -> CLEANED_RESULT_TYPE {
+                return (object->*met)(cpp_args...);
+            };
+            return call<decltype(f), CLEANED_RESULT_TYPE, T...>(f, args);
+        }
+        inline int nargs() { return sizeof...(T); }
+        inline bool is_void() { return std::is_void<RESULT_TYPE>::value; }
+        inline bool is_const() { return IsConst; }
+        inline void signature(std::string& s, const char* name) { Rcpp::signature<RESULT_TYPE,T...>(s, name); }
+    private:
+        Method met;
+    };
+
+    template <typename Class, typename RESULT_TYPE, typename... T>
+    using CppMethodN = CppMethodImplN<false, Class, RESULT_TYPE, T...>;
+
+    template <typename Class, typename RESULT_TYPE, typename... T>
+    using const_CppMethodN = CppMethodImplN<true, Class, RESULT_TYPE, T...>;
+
+    template <bool IsConst, typename Class, typename RESULT_TYPE, typename... T>
+    class Pointer_CppMethodImplN : public CppMethod<Class> {
+    public:
+        typedef typename std::conditional<IsConst, RESULT_TYPE (*)(const Class*, T...),
+                                            RESULT_TYPE (*)(Class*, T...)>::type Method;
+        typedef CppMethod<Class> method_class;
+        typedef typename Rcpp::traits::remove_const_and_reference<RESULT_TYPE>::type CLEANED_RESULT_TYPE;
+
+        Pointer_CppMethodImplN(Method m) : method_class(), met(m) {}
+        SEXP operator()(Class* object, SEXP* args) {
+            // Need to have `object` as the first argument to the function, so wrap it in a lambda
+            auto f = [&object, this](T... cpp_args) -> CLEANED_RESULT_TYPE {
+                return met(object, cpp_args...);
+            };
+            return call<decltype(f), CLEANED_RESULT_TYPE, T...>(f, args);
+        }
+        inline int nargs() { return sizeof...(T); }
+        inline bool is_void() { return std::is_void<RESULT_TYPE>::value; }
+        inline bool is_const() { return IsConst; }
+        inline void signature(std::string& s, const char* name) { Rcpp::signature<RESULT_TYPE,T...>(s, name); }
+    private:
+        Method met;
+    };
+
+    template <typename Class, typename RESULT_TYPE, typename... T>
+    using Pointer_CppMethodN = Pointer_CppMethodImplN<false, Class, RESULT_TYPE, T...>;
+
+    template <typename Class, typename RESULT_TYPE, typename... T>
+    using Const_Pointer_CppMethodN = Pointer_CppMethodImplN<true, Class, RESULT_TYPE, T...>;
+#else
+    #include <Rcpp/module/Module_generated_CppMethod.h>
+    #include <Rcpp/module/Module_generated_Pointer_CppMethod.h>
+#endif
 
     template <typename Class>
     class CppProperty {
@@ -368,8 +552,28 @@ namespace Rcpp{
     } ;
 }
 
-// function factories
-#include <Rcpp/module/Module_generated_function.h>
+#if defined(HAS_VARIADIC_TEMPLATES)
+namespace Rcpp {
+    template <typename RESULT_TYPE, typename... T>
+    void function(const char* name_,  RESULT_TYPE (*fun)(T... t), const char* docstring = 0) {
+        Rcpp::Module* scope = ::getCurrentScope();
+        if (scope) {
+            scope->Add(name_, new CppFunctionN<RESULT_TYPE, T...>(fun, docstring));
+        }
+    }
+
+    template <typename RESULT_TYPE, typename... T>
+    void function(const char* name_,  RESULT_TYPE (*fun)(T... t), Rcpp::List formals, const char* docstring = 0) {
+        Rcpp::Module* scope = ::getCurrentScope();
+        if (scope) {
+            scope->Add(name_, new CppFunction_WithFormalsN<RESULT_TYPE, T...>(fun, formals, docstring));
+        }
+    }
+}
+#else
+    // function factories
+    #include <Rcpp/module/Module_generated_function.h>
+#endif
 
 namespace Rcpp {
 
@@ -454,7 +658,6 @@ static VARIABLE_IS_NOT_USED SEXP moduleSym = NULL;
 // this macro is called by code wanting to load a module -- see RInside's rinside_module_sample0.cpp
 #define LOAD_RCPP_MODULE(NAME) \
     Shield<SEXP> __load_module_call__( Rf_lang2( GET_MODULE_SYM, _rcpp_module_boot_##NAME() ) ); \
-    Rcpp_eval( __load_module_call__, R_GlobalEnv );
+    Rcpp_fast_eval( __load_module_call__, R_GlobalEnv );
 
 #endif
-
